use chrono::Utc;
use gtk::prelude::*;
use relm4::{
    component::{AsyncComponent, AsyncComponentParts},
    prelude::*,
    AsyncComponentSender,
};

use crate::{
    database::Database,
    ui::{mcdu::McduOutput, sheet_model::SheetModelType},
    AppInitData,
};

use super::{
    mcdu::McduModel,
    sheet_listing::{SheetListingInput, SheetListingModel},
};

pub struct AppModel {
    database: Database,
    mcdu: Controller<McduModel>,
    sheets_and_files_listing: Controller<SheetListingModel>,
}

#[derive(Debug)]
pub enum AppInput {
    SearchStarted(String),
    SheetPressed(SheetModelType),
}

#[relm4::component(pub, async)]
impl AsyncComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = AppInitData;
    type CommandOutput = ();

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            set_maximized: true,
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_hexpand: true,
                    gtk::ScrolledWindow {
                            model.sheets_and_files_listing.widget(),
                            set_vexpand: true,
                            set_hexpand: true,
                    },
                },
                model.mcdu.widget(),
            }
        }
    }

    async fn init(
        init_data: Self::Init,
        window: Self::Root,
        sender: AsyncComponentSender<Self>,
    ) -> AsyncComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                McduOutput::SearchStarted(query) => AppInput::SearchStarted(query),
            });

        let mut orphan_files: Vec<SheetModelType> = init_data
            .orphans
            .into_iter()
            .map(|orphan| SheetModelType::Orphan { orphan })
            .collect();
        orphan_files.sort_by(|a, b| a.cmp(b).reverse());

        let mut sheets_and_files: Vec<SheetModelType> = init_data
            .sheets
            .into_iter()
            .map(|sheet| SheetModelType::Sheet { sheet })
            .chain(orphan_files)
            .collect();
        sheets_and_files.sort_by(|a, b| a.cmp(b).reverse());

        let sheets_and_files_listing = SheetListingModel::builder()
            .launch(sheets_and_files)
            .forward(sender.input_sender(), |response| {
                AppInput::SheetPressed(response.sheet_model_type)
            });

        let model = AppModel {
            database: init_data.database,
            mcdu,
            sheets_and_files_listing,
        };

        let widgets = view_output!();

        AsyncComponentParts { model, widgets }
    }

    async fn update(
        &mut self,
        message: Self::Input,
        _sender: AsyncComponentSender<Self>,
        _root: &Self::Root,
    ) {
        // AppInput::SheetPressed(sheet) => opener::open(sheet).unwrap(),
        match message {
            AppInput::SearchStarted(query) => {
                self.sheets_and_files_listing
                    .emit(SheetListingInput::Query(query.clone()));
            }
            AppInput::SheetPressed(sheet_model_type) => {
                opener::open(sheet_model_type.get_path()).unwrap();
                match sheet_model_type {
                    SheetModelType::Orphan { mut orphan } => {
                        orphan.last_opened = Utc::now();
                        self.database
                            .update_orphan_last_opened(&orphan)
                            .await
                            .unwrap();
                    }
                    SheetModelType::Sheet { mut sheet } => {
                        sheet.last_opened = Utc::now();
                        self.database
                            .update_sheet_last_opened(&sheet)
                            .await
                            .unwrap();
                    }
                };
            }
        }
    }
}
